@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2011, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,8 +24,8 @@
 #include "workbench/wb_context_names.h"
 #include "objimpl/db.query/db_query_Resultset.h"
 #include "objimpl/ui/mforms_ObjectReference_impl.h"
-#include "grtui/inserts_export_form.h"
 #include "grtdb/editor_dbobject.h"
+#include "grtdb/db_helpers.h"
 
 #include "sqlide/wb_sql_editor_tree_controller.h"
 #include "sqlide/sql_script_run_wizard.h"
@@ -93,7 +93,7 @@ void SqlEditorForm::limit_rows(mforms::MenuItem *menu, const char *limit)
   if (limit_num > 0)
     _grtm->set_app_option("SqlEditor:LimitRowsCount", grt::IntegerRef(limit_num));
 
-  set_editor_tool_items_checked("query.toggleLimit", (atoi(limit) > 0));
+  set_editor_tool_items_checked("query.toggleLimit", (limit_num > 0));
 
   // special handling for custom values not in the predefined list
   mforms::MenuItem *citem = menu->find_item("custom");
@@ -182,18 +182,24 @@ void SqlEditorForm::update_menu_and_toolbar()
       item->set_enabled(!running && connected);
       item->set_checked(auto_commit());    
     }
-    item = _menu->find_item("query.showFieldInfo");
+    item = _menu->find_item("query.gatherFieldInfo");
     if (item)
     {
       item->set_enabled(!running && connected);
-      item->set_checked(_show_field_info);
+      item->set_checked(collect_field_info());
+    }
+    item = _menu->find_item("query.gatherPSInfo");
+    if (item)
+    {
+      item->set_enabled(!running && connected && bec::is_supported_mysql_version_at_least(_version, 5, 5));
+      item->set_checked(collect_ps_statement_events());
     }
   }
 
   if (_toolbar)
   {
     _toolbar->set_item_enabled("query.new_schema", connected);
-    _toolbar->set_item_enabled("query.show_schema_inspector", connected);
+    _toolbar->set_item_enabled("query.show_inspector", connected);
     _toolbar->set_item_enabled("query.new_table", connected);
     _toolbar->set_item_enabled("query.new_view", connected);
     _toolbar->set_item_enabled("query.new_routine", connected);
@@ -325,7 +331,7 @@ void SqlEditorForm::toolbar_command(const std::string& command)
     _live_tree->tree_create_object(LiveSchemaTree::Procedure, "", "");
   else if (command == "query.new_function")
     _live_tree->tree_create_object(LiveSchemaTree::Function, "", "");
-  else if (command == "query.show_schema_inspector")
+  else if (command == "query.show_inspector")
   {
     db_query_EditorRef editor(_wbsql->get_grt_editor_object(this));
     if (editor.is_valid())
@@ -334,32 +340,36 @@ void SqlEditorForm::toolbar_command(const std::string& command)
       args.ginsert(editor);
 
       grt::ListRef<db_query_LiveDBObject> selection = grt::ListRef<db_query_LiveDBObject>::cast_from(get_live_tree()->get_schema_tree()->get_selected_objects());
-      grt::BaseListRef selected_schemas(_grtm->get_grt());
+      grt::BaseListRef selected_items(_grtm->get_grt());
       GRTLIST_FOREACH (db_query_LiveDBObject, selection, iterator)
       {
         std::string type = (*iterator)->type();
-        if (type == "db.Schema")
-          selected_schemas.ginsert((*iterator)->name());
+        if (type == "db.Schema" || type == "db.Table" || type == "db.Index")
+          selected_items.ginsert((*iterator));
       }
 
-      if (selected_schemas->count() > 0)
+      if (selected_items->count() > 0)
       {
-        args.ginsert(selected_schemas);
+        args.ginsert(selected_items);
         grt::Module *module = _grtm->get_grt()->get_module("SQLIDEUtils");
         if (module)
-          module->call_function("showSchemaInspector", args);
+          module->call_function("showInspector", args);
       }
       else if (!active_schema().empty())
       {
-        selected_schemas.ginsert(grt::StringRef(active_schema()));
-        args.ginsert(selected_schemas);
+        db_query_LiveDBObjectRef obj(_grtm->get_grt());
+        obj->schemaName(active_schema());
+        obj->type("db.Schema");
+        obj->name(active_schema());
+        selected_items.ginsert(obj);
+        args.ginsert(selected_items);
         grt::Module *module = _grtm->get_grt()->get_module("SQLIDEUtils");
         if (module)
-          module->call_function("showSchemaInspector", args);
+          module->call_function("showInspector", args);
       }
       else
         mforms::Utilities::show_warning(_("Selection empty"),
-        _("Select at least one schema to show the inspector"), "Close");
+        _("Select a schema, table or index object in the schema tree to show the inspector."), "Close");
     }
   }
 }
@@ -467,22 +477,10 @@ boost::shared_ptr<mforms::ToolBar> SqlEditorForm::setup_editor_toolbar(Sql_edito
   item->set_name("query.explain_current_statement");
   item->set_icon(IconManager::get_instance()->get_icon_path("qe_sql-editor-tb-icon_explain.png"));
   item->set_tooltip(_("Execute the EXPLAIN command on the statement under the cursor"));
-  bool has_json_explain = false;
-  {
-    int major, minor;
-    if ((sscanf(_connection_details["dbmsProductVersion"].c_str(), "%i.%i", &major, &minor) == 2
-        && major > 5) || (major == 5 && minor >= 6))
-      has_json_explain = true;
-  }
-  if (has_json_explain)
-  {
-    _wbsql->get_cmdui()->scoped_connect(item->signal_activated(),
-                                        boost::bind((void (wb::CommandUI::*)(const std::string&))&wb::CommandUI::activate_command, 
-                                                    _wbsql->get_cmdui(), "plugin:wb.sqlide.visual_explain"));
-  }
-  else
-    scoped_connect(item->signal_activated(),boost::bind(&SqlEditorForm::explain_current_statement, this));
-  tbar->add_item(item);  
+  _wbsql->get_cmdui()->scoped_connect(item->signal_activated(),
+                                      boost::bind((void (wb::CommandUI::*)(const std::string&))&wb::CommandUI::activate_command, 
+                                                  _wbsql->get_cmdui(), "plugin:wb.sqlide.visual_explain"));
+  tbar->add_item(item);
   
   item = mforms::manage(new mforms::ToolBarItem(mforms::ActionItem));
   item->set_name("query.cancel");
@@ -530,7 +528,7 @@ boost::shared_ptr<mforms::ToolBar> SqlEditorForm::setup_editor_toolbar(Sql_edito
   item->set_name("query.autocommit");
   item->set_alt_icon(IconManager::get_instance()->get_icon_path("qe_sql-editor-tb-icon_autocommit-on.png"));
   item->set_icon(IconManager::get_instance()->get_icon_path("qe_sql-editor-tb-icon_autocommit-off.png"));
-  item->set_tooltip(_("Toggle autocommit mode. When enabled, each statement will be commited immediately.\nNOTE: all query tabs in the same connection share the same transaction. To have independent transactions, you must open a new connection."));
+  item->set_tooltip(_("Toggle autocommit mode. When enabled, each statement will be committed immediately.\nNOTE: all query tabs in the same connection share the same transaction. To have independent transactions, you must open a new connection."));
   scoped_connect(item->signal_activated(),boost::bind(&SqlEditorForm::toggle_autocommit, this));
   tbar->add_item(item);  
   
@@ -561,79 +559,6 @@ void SqlEditorForm::set_tool_item_checked(const std::string& name, bool flag)
   _toolbar->set_item_checked(name, flag);
 }
 
-
-
-void SqlEditorForm::show_export_recordset(int editor_index, Recordset::Ptr rs_ptr)
-{
-  try
-  {
-    RETURN_IF_FAIL_TO_RETAIN_WEAK_PTR (Recordset, rs_ptr, rs)
-    {
-      grt::ValueRef option(_grtm->get_app_option("Recordset:LastExportPath"));
-      std::string path = option.is_valid() ? grt::StringRef::cast_from(option) : "";
-      option = _grtm->get_app_option("Recordset:LastExportExtension");
-      std::string extension = option.is_valid() ? grt::StringRef::cast_from(option) : "";
-      InsertsExportForm exporter(0/*mforms::Form::main_form()*/, rs_ref, extension);
-      exporter.set_title(_("Export Resultset"));
-      if (!path.empty())
-        exporter.set_path(path);
-      path = exporter.run();
-      if (path.empty())
-        _grtm->replace_status_text(_("Export resultset canceled"));
-      else
-      {
-        _grtm->replace_status_text(strfmt(_("Exported resultset to %s"), path.c_str()));
-        _grtm->set_app_option("Recordset:LastExportPath", grt::StringRef(path));
-        extension = base::extension(path);
-        if (!extension.empty() && extension[0] == '.')
-          extension = extension.substr(1);
-        if (!extension.empty())
-          _grtm->set_app_option("Recordset:LastExportExtension", grt::StringRef(extension));    
-      }
-    }
-  }
-  catch (const std::exception &exc)
-  {
-    mforms::Utilities::show_error("Error exporting recordset", exc.what(), "OK");
-  }
-}
-
-
-void SqlEditorForm::show_import_recordset(int editor_index, Recordset::Ptr rs_ptr)
-{
-  try
-  {
-    RETURN_IF_FAIL_TO_RETAIN_WEAK_PTR (Recordset, rs_ptr, rs)
-    {
-      grt::BaseListRef args(_grtm->get_grt());
-      
-      db_query_ResultsetRef rset;
-      db_query_QueryEditorRef qeditor(db_query_QueryEditorRef::cast_from(_sql_editors[editor_index]->editor->grtobj()));
-      for (size_t c = qeditor->resultsets().count(), i = 0; i < c; i++)
-      {
-        if (dynamic_cast<WBRecordsetResultset*>(qeditor->resultsets()[i]->get_data())->recordset.get() == rs)
-        {
-          rset = qeditor->resultsets()[i];
-          break;
-        }
-      }
-      
-      if (rset.is_valid())
-      {
-        args.ginsert(rset);
-        grt::Module *module = _grtm->get_grt()->get_module("SQLIDEUtils");
-        if (module)
-          module->call_function("importRecordsetDataFromFile", args);
-      }
-    }
-  }
-  catch (const std::exception &exc)
-  {
-    mforms::Utilities::show_error("Error importing recordset", exc.what(), "OK");
-  }
-}
-
-
 bool SqlEditorForm::run_live_object_alteration_wizard(const std::string &alter_script, bec::DBObjectEditorBE* obj_editor, RowId log_id,
                                                       const std::string &log_context)
 {
@@ -647,9 +572,9 @@ bool SqlEditorForm::run_live_object_alteration_wizard(const std::string &alter_s
   std::string lock;
   wbsql()->get_wbui()->get_wb_options_value("", "DbSqlEditor:OnlineDDLLock", lock);
   
-  SqlScriptRunWizard wizard(_grtm, rdbms()->version(), algorithm, lock);
+  SqlScriptRunWizard wizard(_grtm, rdbms_version(), algorithm, lock);
   if (obj_editor)
-    wizard.regenerate_script = boost::bind(&SqlEditorTreeController::generate_alter_script, get_live_tree(),
+    wizard.regenerate_script = boost::bind(&SqlEditorTreeController::generate_alter_script, get_live_tree(), obj_editor->get_rdbms(),
                                            obj_editor->get_dbobject(), _1, _2);
   scoped_connection c1(on_sql_script_run_error.connect(boost::bind(&SqlScriptApplyPage::on_error, wizard.apply_page, _1, _2, _3)));
   scoped_connection c2(on_sql_script_run_progress.connect(boost::bind(&SqlScriptApplyPage::on_exec_progress, wizard.apply_page, _1)));
