@@ -1320,9 +1320,12 @@ void SqlEditorTreeController::tree_activate_objects(const std::string& action,
 
     if (!sql.empty())
     {
+      bool _autosave = _owner->get_autosave_disabled();
+      _owner->set_autosave_disabled(true);
       int ed = _owner->run_sql_in_scratch_tab(sql, false, true);
       _owner->sql_editor_caption(ed, changes[i].name);
       _owner->do_partial_ui_refresh(SqlEditorForm::RefreshEditorTitle);
+      _owner->set_autosave_disabled(_autosave);
     }
   }
 }
