@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,7 +17,6 @@
  * 02110-1301  USA
  */
 
-
 #include "stdafx.h"
 
 #include "recordset_text_storage.h"
@@ -66,15 +65,16 @@ static void process_templates(const std::list<std::string> &files)
       Recordset_text_storage::TemplateInfo info;
       info.path = base::strip_extension(*f)+".tpl";
       info.name = base::strip_extension(base::basename(*f));
-      info.extension = cf.get_string("extension");
-      info.description = cf.get_string("description");
+      info.extension = cf.get_value("extension");
+      info.description = cf.get_value("description");
       info.pre_quote_strings = cf.get_bool("pre_quote_strings");
-      info.include_column_types = cf.get_string("include_column_types");
-      info.null_syntax = cf.get_string("null_syntax");
-      info.row_separator = cf.get_string("row_separator");
+      info.quote = cf.get_value("quote");
+      info.include_column_types = cf.get_value("include_column_types");
+      info.null_syntax = cf.get_value("null_syntax");
+      info.row_separator = cf.get_value("row_separator");
       if (info.include_column_types != "xls")
         info.include_column_types = "";
-      std::string args = cf.get_string("arguments");
+      std::string args = cf.get_value("arguments");
       std::vector<std::string> arguments = base::split_token_list(args, ';');
       for (std::vector<std::string>::const_iterator arg = arguments.begin(); arg != arguments.end(); ++arg)
       {
@@ -237,6 +237,8 @@ void Recordset_text_storage::do_serialize(const Recordset *recordset, sqlite::co
   ColumnId visible_col_count= recordset->get_column_count();
   sqlide::QuoteVar qv;
   {
+    if (info.quote != "")
+      qv.quote = info.quote;
     qv.escape_string= std::ptr_fun(escape_sql_string_);
     // swap db (sqlite) stores unknown values as quoted strings
     qv.store_unknown_as_string= true;
