@@ -33,8 +33,6 @@
 
 using base::strfmt;
 
-static const char* dbgdomain = "OverviewPanel";
-
 class OverviewDivision : public Gtk::VBox
 {
   bool header_button_release(GdkEventButton *e)
@@ -212,13 +210,12 @@ public:
 
   void add_edit_buttons(const std::string &obj_type)//const sgic::slot<void> &add_slot, const sgic::slot<void> &del_slot)
   {
-    Gtk::Button *btn;
       
-    btn= add_action_button(_header_box, "collapsing_panel_header_tab_add2.png", 
+    add_action_button(_header_box, "collapsing_panel_header_tab_add2.png", 
                            strfmt(_("Add a new %s"), obj_type.c_str()),
                            sigc::mem_fun(this, &OverviewDivision::add_clicked));
     //      btn->set_tooltip_text(_("Add new item"));
-    btn= add_action_button(_header_box, "collapsing_panel_header_tab_del2.png",
+    add_action_button(_header_box, "collapsing_panel_header_tab_del2.png",
                            strfmt(_("Delete the selected %s"), obj_type.c_str()),
                            sigc::mem_fun(this, &OverviewDivision::delete_clicked));
     //      btn->set_tooltip_text(_("Delete selected item"));
@@ -546,7 +543,7 @@ OverviewItemContainer::OverviewItemContainer(wb::OverviewBE *overview, const bec
   if (get_tree_view())
   {
     // create columns for the tree view
-    tv_model->model().append_string_column(wb::OverviewBE::Label, _("Name"), EDITABLE, WITH_ICON);
+    tv_model->model().append_string_column(wb::OverviewBE::Label, _("Name"), EDITABLE_WO_FIRST, WITH_ICON);
     
     int columnCount = _overview->get_details_field_count(node);
     for (int counter = 0; counter < columnCount; counter++)
