@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -310,15 +310,6 @@ static int processTaskFinish(WBSQLQueryPanel *self)
   return 0;
 }
 
-static int processTaskProgress(float progress, const std::string &message, WBSQLQueryPanel *self)
-{
-  [self->mMessagesTable reloadData];
-  [self->mMessagesTable scrollRowToVisible: [self->mMessagesTable numberOfRows]-1];
-  
-  return 0;
-}
-
-
 - (void)refreshTable:(NSTableView*)table
 {
   [table reloadData];
@@ -353,13 +344,13 @@ static void recordsetListChanged(int editor_index, Recordset::Ref rs, bool added
       NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
       [[self queryTabForBackEndIndex: editor_index]
             performSelectorOnMainThread: @selector(removeRecordsetWithIdentifier:) 
-                             withObject: [NSString stringWithFormat:@"rset%lli", rs->key()]
+                             withObject: [NSString stringWithFormat:@"rset%li", rs->key()]
                           waitUntilDone: NO];
       [pool release];
     }
     else
       [[self queryTabForBackEndIndex: editor_index]
-           removeRecordsetWithIdentifier: [NSString stringWithFormat:@"rset%lli", rs->key()]];
+           removeRecordsetWithIdentifier: [NSString stringWithFormat:@"rset%li", rs->key()]];
   }
   else
     [[self queryTabForBackEndIndex: editor_index]
@@ -860,7 +851,7 @@ willCloseTabViewItem:(NSTabViewItem*)tabViewItem
 
       if (appView)
       {
-        deleg->handle_close_clicked(appView);
+        mDockingPoint->close_view(appView);
       }
       else
       {
@@ -1020,14 +1011,13 @@ willCloseTabViewItem:(NSTabViewItem*)tabViewItem
 
     grtm = be->grt_manager();
     mBackEnd= be;
-    mBackEnd->log()->refresh_ui_cb= boost::bind(reloadTable, mMessagesTable, self);
-    mBackEnd->history()->entries_model()->refresh_ui_cb= boost::bind(reloadTable, mHistoryTable, self);
-    mBackEnd->history()->details_model()->refresh_ui_cb= boost::bind(reloadTable, mHistoryDetailsTable, self);
+    mBackEnd->log()->refresh_ui_signal.connect(boost::bind(reloadTable, mMessagesTable, self));
+    mBackEnd->history()->entries_model()->refresh_ui_signal.connect(boost::bind(reloadTable, mHistoryTable, self));
+    mBackEnd->history()->details_model()->refresh_ui_signal.connect(boost::bind(reloadTable, mHistoryDetailsTable, self));
     
     mBackEnd->set_partial_refresh_ui_slot(boost::bind(refreshUIPartial, _1, self));
     mBackEnd->output_text_slot= boost::bind(addTextToOutput, _1, _2, self);
     
-    mBackEnd->exec_sql_task->progress_cb(boost::bind(processTaskProgress, _1, _2, self));
     mBackEnd->exec_sql_task->finish_cb(boost::bind(processTaskFinish, self));
     mBackEnd->recordset_list_changed.connect(boost::bind(recordsetListChanged, _1, _2, _3, self));
     
@@ -1046,7 +1036,7 @@ willCloseTabViewItem:(NSTabViewItem*)tabViewItem
                                    nil] autorelease]];
     mTextOutputLock= [[NSLock alloc] init];
     
-    NSFont *font = [NSFont fontWithName: @"Andale Mono" 
+    NSFont *font = [NSFont fontWithName: @"AndaleMono"
                                    size: [NSFont smallSystemFontSize]];
     if (!font)
       font = [NSFont fontWithName: @"Monaco" size: [NSFont smallSystemFontSize]];
