@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -293,7 +293,7 @@ namespace MySQL.GUI.Workbench
         if (document is MySQL.Forms.AppViewDockContent)
         {
           MySQL.Forms.AppViewDockContent content = document as MySQL.Forms.AppViewDockContent;
-          e.canClose = content.GetAppView().DocumentClosing();
+          e.canClose = content.DocumentClosing();
         }
     }
 
@@ -343,24 +343,28 @@ namespace MySQL.GUI.Workbench
       {
         wbOverview.refresh_node(node, false);
 
-        if (nodeType == (int)Overview.NodeType.Item)
+        switch ((Overview.NodeType)nodeType)
         {
-          // find the object in its container
-          NodeId parent = wbOverview.get_parent(node);
-          if (listsByNode.ContainsKey(parent.repr()))
-          {
-            ListView list = listsByNode[parent.repr()];
-            UpdateListViewNode(node, list);
-          }
-        }
-        else
-        {
-          // find the control for the id
-          if (nodeType == (int)Overview.NodeType.Section)
-          {
+          case Overview.NodeType.Root:
+            // Do nothing. We refresh the entire content with the model refresh notification.
+            break;
+
+          case Overview.NodeType.Item:
+            // find the object in its container
+            NodeId parent = wbOverview.get_parent(node);
+            if (listsByNode.ContainsKey(parent.repr()))
+            {
+              ListView list = listsByNode[parent.repr()];
+              UpdateListViewNode(node, list);
+            }
+            break;
+
+            
+          case Overview.NodeType.Section:
             // a CollapsingPanel
-          }
-          else if (nodeType == (int)Overview.NodeType.Group)
+            break;
+
+          case Overview.NodeType.Group:
           {
             // schema tabs
             if (panelsByNode.ContainsKey(wbOverview.get_parent(node).repr()))
@@ -370,9 +374,11 @@ namespace MySQL.GUI.Workbench
               panel.Refresh();
               panel.Update();
             }
+            break;
           }
-          else
-            throw new Exception("not handled");
+
+          default:
+            throw new Exception("Model overview: invalid node type found");
         }
       }
     }
@@ -915,6 +921,7 @@ namespace MySQL.GUI.Workbench
       sectionListview.KeyDown += new KeyEventHandler(listViewKeyDown);
       sectionListview.MouseUp += new MouseEventHandler(listViewMouseUp);
       sectionListview.ColumnClick += new ColumnClickEventHandler(ListviewColumnClick);
+      sectionListview.Enter += new EventHandler(ListViewEnter);
 
       // Renaming of overview nodes
       sectionListview.LabelEdit = true;
@@ -936,6 +943,23 @@ namespace MySQL.GUI.Workbench
       return sectionListview;
     }
 
+    /// <summary>
+    /// When entering a list view the surrounding scroll panel scrolls it into view such that its
+    /// left upper corner is at the left upper position. This happens only if the listview is larger
+    /// than the visible scroll panel area. Since clicking on the listview requires it to be already
+    /// in view there's no sense in scrolling.
+    /// We try to counter the scroll event by scrolling back to where we were before, even though
+    /// this means slight flickering.
+    /// </summary>
+    void ListViewEnter(object sender, EventArgs e)
+    {
+      Action<Point> d = (Point p) =>
+      {
+        scrollPanel.AutoScrollPosition = new Point(Math.Abs(p.X), Math.Abs(p.Y));
+      };
+      BeginInvoke(d, scrollPanel.AutoScrollPosition);
+    }
+
     void ListviewColumnClick(object sender, ColumnClickEventArgs e)
     {
       ListView listview = (sender as ListView);
