@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -201,8 +201,8 @@ namespace MySQL.GUI.Workbench
           {
             foreach (TabPage resultTabPage in resultTabControl.TabPages)
             {
-              if (resultTabPage.Tag is AppViewImpl)
-                RemoveViewFromResultTabview(editor, resultTabPage.Tag as AppViewImpl);
+              if (resultTabPage.Tag is AppViewDockContent)
+                RemoveViewFromResultTabview(editor, resultTabPage.Tag as AppViewDockContent);
             }
           }
 
@@ -215,8 +215,8 @@ namespace MySQL.GUI.Workbench
         }
 
         // AppView still does not support can_close/close duality.
-        if (page.Tag is AppViewImpl)
-          (page.Tag as AppViewImpl).DocumentClosing();
+        if (page.Tag is AppViewDockContent)
+          (page.Tag as AppViewDockContent).DocumentClosing();
         else
           closeEditorTab(page); // Non-editor tabs.
       }
@@ -607,8 +607,8 @@ namespace MySQL.GUI.Workbench
       else
       {
         // Any docked app view (plugin)?
-        if (e.page.Tag is AppViewImpl)
-          e.canClose = (e.page.Tag as AppViewImpl).DocumentClosing();
+        if (e.page.Tag is AppViewDockContent)
+          e.canClose = (e.page.Tag as AppViewDockContent).DocumentClosing();
         else
         {
           // One of the object editors.
@@ -795,8 +795,8 @@ namespace MySQL.GUI.Workbench
         return;
 
       TabPage page = resultTabControl.SelectedTab;
-      if (page.Tag is AppViewImpl)
-        page.Text = (page.Tag as AppViewImpl).GetTitle();
+      if (page.Tag is AppViewDockContent)
+        page.Text = (page.Tag as AppViewDockContent).GetTitle();
       else
       {
         MySQL.Grt.Db.RecordsetWrapper rs = page2recordset[resultTabControl.SelectedTab];
@@ -810,7 +810,7 @@ namespace MySQL.GUI.Workbench
     }
 
     delegate void DelegateFunc();
-    private void RecordsetListChanged(int editor_index, long key, bool added)
+    private void RecordsetListChanged(Int32 editor_index, Int32 key, bool added)
     {
       if (added)
       {
@@ -828,7 +828,7 @@ namespace MySQL.GUI.Workbench
       }
     }
 
-    private void AddRecordsetToResultTabview(int editorIndex, long key)
+    private void AddRecordsetToResultTabview(Int32 editorIndex, Int32 key)
     {
       if (recordset2page.ContainsKey(key))
         return; // Shouldn't happen. Just in case...
@@ -845,17 +845,26 @@ namespace MySQL.GUI.Workbench
       // remove the busy animation for that (here known) editor.
       mainContentTabControl.SetBusy(TabIndexFromEditor(sqlEditors[editorIndex]), false);
 
-      MySQL.Grt.Db.RecordsetWrapper rs = dbSqlEditorBE.recordset_for_key(editorIndex, key);
+      MySQL.Grt.Db.RecordsetWrapper rs = dbSqlEditorBE.recordset_for_key(editorIndex, (int)key);
       TabPage page = new TabPage(rs.caption());
 
       RecordsetView recordsetView = new RecordsetView();
+      recordsetView.SetupRecordset(rs);
       page.Tag = recordsetView;
 
+      // recordset view is embedded in the result panel and the result panel is added to the resultTabControl
+      dbSqlEditorBE.dock_grid_to_result_panel(recordsetView.GridView, rs);
+
       Form mainForm = ParentForm;
       if (!mainForm.Visible)
         pendingRelayouts.Add(recordsetView);
 
-      recordsetView.Embed(page, rs);
+      // dock result panel in the tab
+      Control panel = dbSqlEditorBE.get_result_panel_for(rs);
+      panel.Parent = page;
+      panel.Dock = DockStyle.Fill;
+      page.Controls.Add(panel);
+
       resultTabControl.TabPages.Add(page);
       int newIndex = resultTabControl.TabPages.Count - 1;
       resultTabControl.SetCloseButtonVisibility(newIndex, FlatTabControl.CloseButtonVisiblity.ShowButton);
@@ -943,8 +952,8 @@ namespace MySQL.GUI.Workbench
       else
       {
         // Any docked app view (plugin)?
-        if (e.page.Tag is AppViewImpl)
-          e.canClose = (e.page.Tag as AppViewImpl).DocumentClosing();
+        if (e.page.Tag is AppViewDockContent)
+          e.canClose = (e.page.Tag as AppViewDockContent).DocumentClosing();
       }
       Logger.LogDebug("WQE.net", 1, "Can close: " + e.canClose + "\n");
     }
@@ -961,7 +970,7 @@ namespace MySQL.GUI.Workbench
       MySQL.Grt.Db.RecordsetWrapper rs = page2recordset.ContainsKey(e.page) ? page2recordset[e.page] : null;
       if (rs != null)
         rs.close(); // Will trigger the notification chain via RecordListChanged.
-      // No need for handling of AppViewImpl, as this is already closed in ResultTabClosing.
+      // No need for handling of AppViewDockContent, as this is already closed in ResultTabClosing.
     }
 
     private void recordsetTabContextMenuClick(object sender, EventArgs e)
@@ -1238,16 +1247,14 @@ namespace MySQL.GUI.Workbench
       loadSelectedHistoryItems(HistoryAction.HistoryAppendToEditor);
     }
 
-    private int ProcessModelHistoryEntryRowsChange()
+    private void ProcessModelHistoryEntryRowsChange()
     {
       Logger.LogDebug("WQE.net", 1, "Processing history entry changes\n");
-      int ret_val = historyEntriesView.ProcessModelRowsChange();
+      historyEntriesView.ProcessModelRowsChange();
       int selected_entry = dbSqlEditorBE.history().current_entry();
 
       if (selected_entry != -1)
         historyEntriesView.SetRowSelected(selected_entry);
-
-      return ret_val;
     }
 
     #endregion
@@ -1524,7 +1531,7 @@ namespace MySQL.GUI.Workbench
         }
         else if (document is MySQL.Forms.AppViewDockContent)
         {
-          if (!(document as MySQL.Forms.AppViewDockContent).GetAppView().DocumentClosing())
+          if (!(document as MySQL.Forms.AppViewDockContent).DocumentClosing())
             return false;
         }
       }
@@ -1537,14 +1544,14 @@ namespace MySQL.GUI.Workbench
     /// </summary>
     /// <param name="editor">The editor whose result tabview is used.</param>
     /// <param name="view">The view to dock.</param>
-    public void AddViewToResultTabview(SqlEditorWrapper editor, AppViewImpl view)
+    public void AddViewToResultTabview(SqlEditorWrapper editor, AppViewDockContent view)
     {
       Logger.LogDebug("WQE.net", 1, "Adding new view to result tabview\n");
       FlatTabControl resultTabControl = GetResultTabControlForEditor(editor);
       if (resultTabControl == null)
         return;
 
-      int newIndex = resultTabControl.AddDocument(view.GetHost());
+      int newIndex = resultTabControl.AddDocument(view);
       resultTabControl.TabPages[newIndex].Tag = view;
       resultTabControl.SetCloseButtonVisibility(newIndex, FlatTabControl.CloseButtonVisiblity.ShowButton);
       resultTabControl.SelectedIndex = newIndex;
@@ -1558,14 +1565,14 @@ namespace MySQL.GUI.Workbench
     /// </summary>
     /// <param name="editor">The editor whose result tabview is used.</param>
     /// <param name="view">The view to dock.</param>
-    public void RemoveViewFromResultTabview(SqlEditorWrapper editor, AppViewImpl view)
+    public void RemoveViewFromResultTabview(SqlEditorWrapper editor, AppViewDockContent view)
     {
       Logger.LogDebug("WQE.net", 1, "Removing view from result tabview\n");
       FlatTabControl resultTabControl = GetResultTabControlForEditor(editor);
       if (resultTabControl == null)
         return;
 
-      TabPage page = view.GetHost().Parent as TabPage;
+      TabPage page = view.Parent as TabPage;
       if (page == null)
         return;
 
@@ -1575,18 +1582,13 @@ namespace MySQL.GUI.Workbench
       page.Dispose();
     }
 
-    public void GetResultTabViewSize(SqlEditorWrapper editor, out int width, out int height)
+    public Size GetResultTabViewSize(SqlEditorWrapper editor)
     {
       FlatTabControl resultTabControl = GetResultTabControlForEditor(editor);
       if (resultTabControl == null)
-      {
-        width = 0;
-        height = 0;
-        return;
-      }
+        return new Size(0, 0);
 
-      width = resultTabControl.ClientSize.Width;
-      height = resultTabControl.ClientSize.Height;
+      return resultTabControl.ClientSize;
     }
 
     /// <summary>
@@ -1610,7 +1612,7 @@ namespace MySQL.GUI.Workbench
 
   #region DockingPoint delegate implementation
 
-  public class ResultPaneDockDelegate : DockingPointDelegateWrapper
+  public class ResultPaneDockDelegate : ManagedDockDelegate
   {
     private SqlIdeForm owner = null;
 
@@ -1630,43 +1632,42 @@ namespace MySQL.GUI.Workbench
       return "db.Query.QueryEditor:result"; // Same as defined in wb_sql_editor_form.h
     }
 
-    public override void dock_view(Object representedObject, AppViewImpl view, String arg1, int arg2)
+    public override void dock_view(Object representedObject, AppViewDockContent view, String arg1, int arg2)
     {
       SqlEditorWrapper wrapper = representedObject as SqlEditorWrapper;
       owner.AddViewToResultTabview(wrapper, view);
     }
 
-    public override void undock_view(Object representedObject, AppViewImpl view)
+    public override void undock_view(Object representedObject, AppViewDockContent view)
     {
       SqlEditorWrapper wrapper = representedObject as SqlEditorWrapper;
       owner.RemoveViewFromResultTabview(wrapper, view);
     }
 
-    public override bool select_view(Object representedObject, AppViewImpl view)
+    public override bool select_view(Object representedObject, AppViewDockContent view)
     {
-      view.GetHost().Activate();
+      view.Activate();
       return true;
     }
 
-    public override void set_view_title(Object representedObject, AppViewImpl view, String str)
+    public override void set_view_title(Object representedObject, AppViewDockContent view, String str)
     {
-      view.GetHost().TabText = str;
+      view.TabText = str;
     }
 
     /// <summary>
     /// Meant to return the size of the host of the view, not the view itself.
     /// </summary>
-    public override void get_size(Object representedObject, out int w, out int h)
+    public override Size get_size(Object representedObject)
     {
       SqlEditorWrapper wrapper = representedObject as SqlEditorWrapper;
-      owner.GetResultTabViewSize(wrapper, out w, out h);
+      return owner.GetResultTabViewSize(wrapper);
     }
   };
 
-  public class MainPaneDockDelegate : DockingPointDelegateWrapper
+  public class MainPaneDockDelegate : ManagedDockDelegate
   {
     private SqlIdeForm owner = null;
-    private TabDocument document;
 
     public MainPaneDockDelegate(SqlIdeForm owner)
       : base(null)
@@ -1684,36 +1685,34 @@ namespace MySQL.GUI.Workbench
       return "db.query.Editor:main"; // Same as defined in wb_sql_editor_form.h
     }
 
-    public override void dock_view(Object representedObject, AppViewImpl view, String arg1, int arg2)
+    public override void dock_view(Object representedObject, AppViewDockContent view, String arg1, int arg2)
     {
-      document = view.GetHost(); // Only for get_size, since the view is not passed in there.
-      TabPage page = owner.DockDocument(document, true);
+      TabPage page = owner.DockDocument(view, true);
       page.Tag = view;
     }
 
-    public override void undock_view(Object representedObject, AppViewImpl view)
+    public override void undock_view(Object representedObject, AppViewDockContent view)
     {
-      owner.UndockDocument(view.GetHost());
+      owner.UndockDocument(view);
     }
 
-    public override bool select_view(Object representedObject, AppViewImpl view)
+    public override bool select_view(Object representedObject, AppViewDockContent view)
     {
-      view.GetHost().Activate();
+      view.Activate();
       return true;
     }
 
-    public override void set_view_title(Object representedObject, AppViewImpl view, String str)
+    public override void set_view_title(Object representedObject, AppViewDockContent view, String str)
     {
-      view.GetHost().TabText = str;
+      view.TabText = str;
     }
 
     /// <summary>
     /// Meant to return the size of the host of the view, not the view itself.
     /// </summary>
-    public override void get_size(Object representedObject, out int w, out int h)
+    public override Size get_size(Object representedObject)
     {
-      w = document.Width;
-      h = document.Height;
+      return new Size(0, 0);
     }
   };
 
