@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2012, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,7 +16,7 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301  USA
  */
-
+#define NOMINMAX
 #include "base/string_utilities.h"
 #include "base/file_functions.h"
 #include "base/log.h"
@@ -33,6 +33,246 @@
 
 DEFAULT_LOG_DOMAIN(DOMAIN_BASE);
 
+
+
+// updated as of 5.7
+static const char *reserved_keywords[] = {
+  "ACCESSIBLE",
+  "ADD",
+  "ALL",
+  "ALTER",
+  "ANALYZE",
+  "AND",
+  "AS",
+  "ASC",
+  "ASENSITIVE",
+  "BEFORE",
+  "BETWEEN",
+  "BIGINT",
+  "BINARY",
+  "BLOB",
+  "BOTH",
+  "BY",
+  "CALL",
+  "CASCADE",
+  "CASE",
+  "CHANGE",
+  "CHAR",
+  "CHARACTER",
+  "CHECK",
+  "COLLATE",
+  "COLUMN",
+  "CONDITION",
+  "CONSTRAINT",
+  "CONTINUE",
+  "CONVERT",
+  "CREATE",
+  "CROSS",
+  "CURRENT_DATE",
+  "CURRENT_TIME",
+  "CURRENT_TIMESTAMP",
+  "CURRENT_USER",
+  "CURSOR",
+  "DATABASE",
+  "DATABASES",
+  "DAY_HOUR",
+  "DAY_MICROSECOND",
+  "DAY_MINUTE",
+  "DAY_SECOND",
+  "DEC",
+  "DECIMAL",
+  "DECLARE",
+  "DEFAULT",
+  "DELAYED",
+  "DELETE",
+  "DESC",
+  "DESCRIBE",
+  "DETERMINISTIC",
+  "DISTINCT",
+  "DISTINCTROW",
+  "DIV",
+  "DOUBLE",
+  "DROP",
+  "DUAL",
+  "EACH",
+  "ELSE",
+  "ELSEIF",
+  "ENCLOSED",
+  "ESCAPED",
+  "EXISTS",
+  "EXIT",
+  "EXPLAIN",
+  "FALSE",
+  "FETCH",
+  "FLOAT",
+  "FLOAT4",
+  "FLOAT8",
+  "FOR",
+  "FORCE",
+  "FOREIGN",
+  "FROM",
+  "FULLTEXT",
+  "GET",
+  "GRANT",
+  "GROUP",
+  "HAVING",
+  "HIGH_PRIORITY",
+  "HOUR_MICROSECOND",
+  "HOUR_MINUTE",
+  "HOUR_SECOND",
+  "IF",
+  "IGNORE",
+  "IN",
+  "INDEX",
+  "INFILE",
+  "INNER",
+  "INOUT",
+  "INSENSITIVE",
+  "INSERT",
+  "INT",
+  "INT1",
+  "INT2",
+  "INT3",
+  "INT4",
+  "INT8",
+  "INTEGER",
+  "INTERVAL",
+  "INTO",
+  "IO_AFTER_GTIDS",
+  "IO_BEFORE_GTIDS",
+  "IS",
+  "ITERATE",
+  "JOIN",
+  "KEY",
+  "KEYS",
+  "KILL",
+  "LEADING",
+  "LEAVE",
+  "LEFT",
+  "LIKE",
+  "LIMIT",
+  "LINEAR",
+  "LINES",
+  "LOAD",
+  "LOCALTIME",
+  "LOCALTIMESTAMP",
+  "LOCK",
+  "LONG",
+  "LONGBLOB",
+  "LONGTEXT",
+  "LOOP",
+  "LOW_PRIORITY",
+  "MASTER_BIND",
+  "MASTER_SSL_VERIFY_SERVER_CERT",
+  "MATCH",
+  "MAXVALUE",
+  "MEDIUMBLOB",
+  "MEDIUMINT",
+  "MEDIUMTEXT",
+  "MIDDLEINT",
+  "MINUTE_MICROSECOND",
+  "MINUTE_SECOND",
+  "MOD",
+  "MODIFIES",
+  "NATURAL",
+  "NONBLOCKING",
+  "NOT",
+  "NO_WRITE_TO_BINLOG",
+  "NULL",
+  "NUMERIC",
+  "ON",
+  "OPTIMIZE",
+  "OPTION",
+  "OPTIONALLY",
+  "OR",
+  "ORDER",
+  "OUT",
+  "OUTER",
+  "OUTFILE",
+  "PARTITION",
+  "PRECISION",
+  "PRIMARY",
+  "PROCEDURE",
+  "PURGE",
+  "RANGE",
+  "READ",
+  "READS",
+  "READ_WRITE",
+  "REAL",
+  "REFERENCES",
+  "REGEXP",
+  "RELEASE",
+  "RENAME",
+  "REPEAT",
+  "REPLACE",
+  "REQUIRE",
+  "RESIGNAL",
+  "RESTRICT",
+  "RETURN",
+  "REVOKE",
+  "RIGHT",
+  "RLIKE",
+  "SCHEMA",
+  "SCHEMAS",
+  "SECOND_MICROSECOND",
+  "SELECT",
+  "SENSITIVE",
+  "SEPARATOR",
+  "SET",
+  "SHOW",
+  "SIGNAL",
+  "SMALLINT",
+  "SPATIAL",
+  "SPECIFIC",
+  "SQL",
+  "SQLEXCEPTION",
+  "SQLSTATE",
+  "SQLWARNING",
+  "SQL_BIG_RESULT",
+  "SQL_CALC_FOUND_ROWS",
+  "SQL_SMALL_RESULT",
+  "SSL",
+  "STARTING",
+  "STRAIGHT_JOIN",
+  "TABLE",
+  "TERMINATED",
+  "THEN",
+  "TINYBLOB",
+  "TINYINT",
+  "TINYTEXT",
+  "TO",
+  "TRAILING",
+  "TRIGGER",
+  "TRUE",
+  "UNDO",
+  "UNION",
+  "UNIQUE",
+  "UNLOCK",
+  "UNSIGNED",
+  "UPDATE",
+  "USAGE",
+  "USE",
+  "USING",
+  "UTC_DATE",
+  "UTC_TIME",
+  "UTC_TIMESTAMP",
+  "VALUES",
+  "VARBINARY",
+  "VARCHAR",
+  "VARCHARACTER",
+  "VARYING",
+  "WHEN",
+  "WHERE",
+  "WHILE",
+  "WITH",
+  "WRITE",
+  "XOR",
+  "YEAR_MONTH",
+  "ZEROFILL",
+  NULL
+};
+
+
 namespace base {
 
 #ifdef _WIN32
@@ -779,6 +1019,31 @@ std::string make_valid_filename(const std::string &name)
   return result;
 }
 
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * Get a string containing the 'len' left most characters.
+ */
+std::string left(const std::string& s, unsigned int len)
+{
+  return s.substr(0, len);
+}
+
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * Get a string containing the 'len' right most characters.
+ */
+std::string right(const std::string& s, unsigned int len)
+{
+  // If len is negative or zero, then an empty string is returned
+  if (len < 1)
+    return "";
+  
+  return s.substr(std::max((int)(s.length() - len), 0));
+}
+
+
 //--------------------------------------------------------------------------------------------------
 
 /**
@@ -786,11 +1051,27 @@ std::string make_valid_filename(const std::string &name)
  */
 bool starts_with(const std::string& s, const std::string& part)
 {
-  return (s.substr(0, part.length()) == part);
+  return s.compare(0, part.length(), part) == 0;
 }
 
 //--------------------------------------------------------------------------------------------------
 
+/**
+ * Tests if t ends with part.
+ */
+bool ends_with(const std::string& s, const std::string& part)
+{
+  int start_at = s.length() - part.length();
+  
+  // If start_at < 0 then the search string is bigger then the source, so the results is false.
+  // On the other hand, if it starts after the end, something went wrong...
+  if (start_at < 0 || start_at > (int)s.length())
+    return false;
+  
+  return s.compare(start_at, std::string::npos, part) == 0;
+}
+//--------------------------------------------------------------------------------------------------
+
 void replace(std::string& value, const std::string& search, const std::string& replacement)
 {
   std::string::size_type next;
@@ -1157,21 +1438,26 @@ std::string quote_identifier(const std::string& identifier, const char quote_cha
  */
 std::string quote_identifier_if_needed(const std::string &ident, const char quote_char)
 {
-  bool needs_quotation= false;
-  int digits = 0;
-  for (std::string::const_iterator i= ident.begin(); i != ident.end(); ++i)
+  bool needs_quotation= is_reserved_word(ident);  // check whether it's a reserved keyword
+  size_t digits = 0;
+
+  if (!needs_quotation)
   {
-    if ((*i >= 'a' && *i <= 'z') || (*i >= 'A' && *i <= 'Z') || (*i >= '0' && *i <= '9')
-      || (*i == '_') || (*i == '$') || ((unsigned char)(*i) > 0x7F))
+    for (std::string::const_iterator i= ident.begin(); i != ident.end(); ++i)
     {
-      if (*i >= '0' && *i <= '9')
-        digits++;
-      
-      continue;
+      if ((*i >= 'a' && *i <= 'z') || (*i >= 'A' && *i <= 'Z') || (*i >= '0' && *i <= '9')
+          || (*i == '_') || (*i == '$') || ((unsigned char)(*i) > 0x7F))
+      {
+        if (*i >= '0' && *i <= '9')
+          digits++;
+
+        continue;
+      }
+      needs_quotation = true;
+      break;
     }
-    needs_quotation = true;
-    break;
   }
+
   if (needs_quotation || digits == ident.length())
     return quote_char + ident + quote_char;
   else
@@ -1298,6 +1584,19 @@ bool contains_string(const std::string &text, const std::string &candidate, bool
 
 //--------------------------------------------------------------------------------------------------
 
+bool is_reserved_word(const std::string &word)
+{
+    std::string upper = base::toupper(word);
+    for (const char **kw = reserved_keywords; *kw != NULL; ++kw)
+    {
+      if (upper.compare(*kw) == 0)
+        return true;
+    }
+    return false;
+}
+
+//--------------------------------------------------------------------------------------------------
+
 EolHelpers::Eol_format EolHelpers::detect(const std::string &text)
 {
   std::string::size_type pos= text.find_first_of("\r\n");
@@ -1455,7 +1754,6 @@ std::string reflow_text(const std::string &text, unsigned int line_length, const
 
   const char *char_string = text.c_str();
   const char *iter = char_string;
-  const char *line_begin = char_string;
 
   unsigned int space_position_source = 0;
   unsigned int line_char_counter = 0;
@@ -1487,7 +1785,6 @@ std::string reflow_text(const std::string &text, unsigned int line_length, const
         result += new_line;
 
         space_position_source += char_count_after_space;
-        line_begin = (char *)char_string + space_position_source;
         line_char_counter = char_count_after_space = 0;
       }
       else
@@ -1498,9 +1795,6 @@ std::string reflow_text(const std::string &text, unsigned int line_length, const
         //  Insert a \n in the right position, right after the space char(or at the end of the string)
         result.size() == break_position ? result += new_line : result.insert(break_position, new_line);
 
-        //  Set the begining of the line of the source
-        line_begin = (char *)char_string + space_position_source;
-
         //  Mark the characters that were already inserted after the new line
         line_char_counter = char_count_after_space;
       }
