@@ -1072,7 +1072,7 @@ bool TreeNodeViewImpl::ColumnRecord::on_focus_out(GdkEventFocus *event, Gtk::Ent
 //---------------------------------------------------------------------------------------
 
 TreeNodeViewImpl::TreeNodeViewImpl(TreeNodeView *self, mforms::TreeOptions opts)
-  : ViewImpl(self), _row_height(-1)
+  : ViewImpl(self), _row_height(-1), _org_event(0)
 {
   _drag_in_progress = false;
   _drag_button = 0;
@@ -1092,9 +1092,10 @@ TreeNodeViewImpl::TreeNodeViewImpl(TreeNodeView *self, mforms::TreeOptions opts)
   _tree.signal_test_expand_row().connect(sigc::bind_return(sigc::mem_fun(this, &TreeNodeViewImpl::on_will_expand), false));
   _tree.signal_button_press_event().connect(sigc::mem_fun(this, &TreeNodeViewImpl::on_button_event), false);
 //  _tree.set_reorderable((opts & mforms::TreeAllowReorderRows) || (opts & mforms::TreeCanBeDragSource)); // we need this to have D&D working
+  _tree.signal_button_release_event().connect(sigc::mem_fun(this, &TreeNodeViewImpl::on_button_release), false);
   if (opts & mforms::TreeCanBeDragSource)
   {
-    _tree.signal_button_release_event().connect(sigc::mem_fun(this, &TreeNodeViewImpl::on_button_release), false);
+
     _tree.signal_motion_notify_event().connect(sigc::mem_fun(this, &TreeNodeViewImpl::on_motion_notify), false);
 
     Gtk::Widget *w = this->get_outer();
@@ -1113,12 +1114,22 @@ TreeNodeViewImpl::TreeNodeViewImpl(TreeNodeView *self, mforms::TreeOptions opts)
   _swin.show_all();
   _tree.set_headers_visible((opts & mforms::TreeNoHeader) == 0);
 }
+TreeNodeViewImpl::~TreeNodeViewImpl()
+{
+  if (_org_event)
+    delete _org_event;
+}
 
 void TreeNodeViewImpl::slot_drag_end(const Glib::RefPtr<Gdk::DragContext> &context)
 {
   ViewImpl::slot_drag_end(context);
   _drag_in_progress  = false;
   _drag_button = 0;
+  if (_org_event)
+  {
+    delete _org_event;
+    _org_event = 0;
+  }
 }
 
 bool TreeNodeViewImpl::slot_drag_failed(const Glib::RefPtr<Gdk::DragContext> &context,Gtk::DragResult result)
@@ -1126,6 +1137,11 @@ bool TreeNodeViewImpl::slot_drag_failed(const Glib::RefPtr<Gdk::DragContext> &co
   bool ret_val = ViewImpl::slot_drag_failed(context, result);
   _drag_in_progress  = false;
   _drag_button = 0;
+  if (_org_event)
+  {
+    delete _org_event;
+    _org_event = 0;
+  }
   return ret_val;
 
 }
@@ -1142,69 +1158,97 @@ bool TreeNodeViewImpl::on_motion_notify(GdkEventMotion *ev)
   {
     if (w->drag_check_threshold(_drag_start_x, _drag_start_y, ev->x, ev->y))
     {
+      if (_org_event != 0)
+      {
+        delete _org_event;
+        _org_event = 0;
+      }
+
+      {
+        //Because of problems when Treeview has been set to multiselect,
+        //there are some DnD problems, below code is fixing those.
+        //We check if users already selected something,
+        //if he is trying to drag not selected row,
+        //we change selection.
+        std::vector<Gtk::TreePath> path_selection;
+        path_selection = _tree.get_selection()->get_selected_rows();
+
+        Gtk::TreeModel::Path selpath;
+        if(_tree.get_path_at_pos(_drag_start_x, _drag_start_y, selpath))
+        {
+          bool found = false;
+          for(std::vector<Gtk::TreePath>::iterator it = path_selection.begin(); it != path_selection.end(); ++it)
+          {
+            if (selpath == *it)
+            {
+              found = true;
+              break;
+            }
+          }
+          if (!found)
+          {
+            //Old selection is invalid, we need to create new one
+            _tree.get_selection()->unselect_all();
+            _tree.get_selection()->select(selpath);
+          }
+        }
+      }
+
+
       TreeNodeView *view = dynamic_cast<TreeNodeView*>(owner);
       if (view)
       {
         mforms::DragDetails details;
-        void *data = NULL;
-        std::string format;
-        if(view->get_drag_data(details, &data, format))
+
+        Gtk::TreeModel::Path path;
+        if(_tree.get_path_at_pos(_drag_start_x, _drag_start_y, path))
         {
-          std::vector<Gtk::TargetEntry> targets;
-          targets.push_back(Gtk::TargetEntry(format,Gtk::TargetFlags(0),0));
+          Glib::RefPtr<Gdk::Pixmap> row_icon = _tree.create_row_drag_icon(path);
+          int ico_w, ico_h;
+          row_icon->get_size(ico_w, ico_h);
 
-          _tree.enable_model_drag_source(targets); // we need this to have D&D working
-          _drag_in_progress = true;
-          details.location = base::Point(ev->x, ev->y);
-          details.allowedOperations = mforms::DragOperationCopy;
-          TreeNodeRef node;
-
-          Gtk::TreeModel::Path path;
-          if(_tree.get_path_at_pos(_drag_start_x, _drag_start_y, path))
-          {
-            //let's make better drag image,
-            Glib::RefPtr<Gdk::Pixmap> row_icon = _tree.create_row_drag_icon(path);
-
-            int ico_w, ico_h;
-            row_icon->get_size(ico_w, ico_h);
-            Glib::RefPtr<Gdk::Pixbuf> pix = Gdk::Pixbuf::create(row_icon->get_image(0, 0, ico_w, ico_h), 0, 0, ico_w, ico_h);
-            //sadly we need a cairo_surface
-            details.image = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, ico_w, ico_h);
-            cairo_t *cr = cairo_create(details.image);
-            gdk_cairo_set_source_pixbuf(cr, pix->gobj(), 0, 0);
-            cairo_paint(cr);
-
-            mforms::DragOperation operation = view->do_drag_drop(details, data, format);
-            cairo_surface_destroy(details.image);
-            cairo_destroy(cr);
-            view->drag_finished(operation);
-
-          }
-        }
-        else
-        {
-          details.location = base::Point(ev->x, ev->y);
-          details.allowedOperations = mforms::DragOperationCopy;
-          format = "STRING";
-          std::list<mforms::TreeNodeRef> selection = view->get_selection();
+          Glib::RefPtr<Gdk::Pixbuf> pix = Gdk::Pixbuf::create(row_icon->get_image(0, 0, ico_w, ico_h), 0, 0, ico_w, ico_h);
+          //sadly we need a cairo_surface
+          details.image = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, ico_w, ico_h);
+          cairo_t *cr = cairo_create(details.image);
+          gdk_cairo_set_source_pixbuf(cr, pix->gobj(), 0, 0);
+          cairo_paint(cr);
 
+          void *data = NULL;
           std::string text;
-          for (std::list<mforms::TreeNodeRef>::const_iterator iterator = selection.begin(); iterator != selection.end(); ++iterator)
+          std::string format;
+          details.location = base::Point(ev->x, ev->y);
+          details.allowedOperations = mforms::DragOperationCopy;
+          if(!view->get_drag_data(details, &data, format))
           {
-                if (!(*iterator)->get_string(0).empty())
-                {
-                  if (text.size() > 0)
-                    text.append(", ");
-                  text.append((*iterator)->get_string(0));
-                }
+
+            format = "STRING";
+            std::list<mforms::TreeNodeRef> selection = view->get_selection();
+
+            for (std::list<mforms::TreeNodeRef>::const_iterator iterator = selection.begin(); iterator != selection.end(); ++iterator)
+            {
+                  if (!(*iterator)->get_string(0).empty())
+                  {
+                    if (text.size() > 0)
+                      text.append(", ");
+                    text.append((*iterator)->get_string(0));
+                  }
+            }
+
+            if (text.empty())
+            {
+              cairo_surface_destroy(details.image);
+              cairo_destroy(cr);
+              return false;
+            }
+
+            data = &text;
           }
 
-          if (text.empty())
-            return false;
-
-          data = &text;
-
-          view->drag_finished(view->do_drag_drop(details, data, format));
+          mforms::DragOperation operation = view->do_drag_drop(details, data, format);
+          cairo_surface_destroy(details.image);
+          cairo_destroy(cr);
+          view->drag_finished(operation);
         }
       }
     }
@@ -1219,6 +1263,16 @@ bool TreeNodeViewImpl::on_button_release(GdkEventButton* ev)
     return false;
   }
 
+  //Because of problems when Treeview has been set to multiselect,
+  //there are some DnD problems, below code is fixing those.
+  //we need this to emit press event again cause we're changing default behavior of it
+  if (_org_event != 0)
+  {
+    gtk_propagate_event((GtkWidget*)_tree.gobj(), (GdkEvent*)_org_event);
+    delete _org_event;
+    _org_event = 0;
+  }
+//  _tree.
   _drag_button = 0;
   return false;
 }
@@ -1235,9 +1289,12 @@ void TreeNodeViewImpl::set_back_color(const std::string &color)
 {
   if (!force_sys_colors)
   {
-    Gdk::Color col(color);
-    _tree.get_colormap()->alloc_color(col);
-    _tree.modify_base(Gtk::STATE_NORMAL, col);
+    if (!color.empty())
+    {
+      Gdk::Color col(color);
+      _tree.get_colormap()->alloc_color(col);
+      _tree.modify_base(Gtk::STATE_NORMAL, col);
+    }
   }
 }
 
@@ -1310,7 +1367,9 @@ bool TreeNodeViewImpl::on_button_event(GdkEventButton *event)
     mforms::TreeNodeView* tv = dynamic_cast<mforms::TreeNodeView*>(owner); 
     if (tv->get_context_menu())
     {
-      tv->get_context_menu()->popup_at(event->x, event->y);
+      Gtk::Menu* m = dynamic_cast<Gtk::Menu*>((Gtk::Widget*)(tv->get_context_menu()->get_data_ptr()));
+      if (m)
+        m->popup(event->button, event->time);
     }
     
     std::list<TreeNodeRef> selected_nodes = this->get_selection(tv);
@@ -1327,9 +1386,14 @@ bool TreeNodeViewImpl::on_button_event(GdkEventButton *event)
   }
   else if (event->button == 1 && _drag_button == 0)
   {
-    _drag_button = event->button;
-    _drag_start_x = event->x;
-    _drag_start_y = event->y;
+    if (_org_event == 0)
+    {
+      _org_event = new GdkEventButton(*event);
+      _drag_button = event->button;
+      _drag_start_x = event->x;
+      _drag_start_y = event->y;
+      ret_val = true;
+    }
   }
 
   
@@ -1541,23 +1605,59 @@ void TreeNodeViewImpl::set_selected(TreeNodeView* self, TreeNodeRef node, bool f
   tree->_conn.unblock();
 }
 
-static int str_number_cmp(const Gtk::TreeModel::iterator& it1, const Gtk::TreeModel::iterator& it2, Gtk::TreeModelColumn< Glib::ustring> *col)
+template<typename T>
+int column_value_compare(const T &val1, const T &val2)
 {
-  double v1 = mforms::TreeNodeView::parse_string_with_unit((*it1).get_value(*col).c_str());
-  double v2 = mforms::TreeNodeView::parse_string_with_unit((*it2).get_value(*col).c_str());
-
-  return (int)(v2-v1);
+  int result = 0;
+  if (val2 < val1) result = -1;
+  else if (val2 > val1) result = 1;
+  return result;
 }
 
-static int str_cmp(const Gtk::TreeModel::iterator& it1, const Gtk::TreeModel::iterator& it2, Gtk::TreeModelColumn< Glib::ustring> *col)
+template<typename T>
+int column_numeric_compare(const Gtk::TreeModel::iterator& it1, const Gtk::TreeModel::iterator& it2, Gtk::TreeModelColumn<T> *col)
 {
-  const Glib::ustring s1 = (*it1).get_value(*col);
-  const Glib::ustring s2 = (*it2).get_value(*col);
+  return column_value_compare((*it1).get_value(*col), (*it2).get_value(*col));
+}
 
-  if (s1.is_ascii() && s2.is_ascii())
-    return strcmp(s1.c_str(), s2.c_str());
-  else
-    return s1.compare(s2);
+int column_string_compare(const Gtk::TreeModel::iterator& it1, const Gtk::TreeModel::iterator& it2, Gtk::TreeModelColumn<Glib::ustring> *col, int type)
+{
+  int result = 0;
+  
+  switch (type)
+  {
+    case StringColumnType:
+    case StringLTColumnType:
+    case IconColumnType:
+    {
+      std::string val1 = (*it1).get_value(*col).c_str();
+      std::string val2 = (*it2).get_value(*col).c_str();
+      result = base::string_compare(val2, val1, false);
+      break;
+    }
+    case IntegerColumnType:
+    case LongIntegerColumnType:
+    {
+      std::istringstream ss_val1((*it1).get_value(*col).c_str());
+      std::istringstream ss_val2((*it2).get_value(*col).c_str());
+      int64_t val1 = 0;
+      int64_t val2 = 0;
+      ss_val1 >> val1;
+      ss_val2 >> val2;
+      result = column_value_compare(val1, val2);
+      break;
+    }
+    case FloatColumnType:
+    case NumberWithUnitColumnType:
+    {
+      double val1 = mforms::TreeNodeView::parse_string_with_unit((*it1).get_value(*col).c_str());
+      double val2 = mforms::TreeNodeView::parse_string_with_unit((*it2).get_value(*col).c_str());
+      result = column_value_compare(val1, val2);
+      break;
+    }
+  }
+  
+  return result;
 }
 
 void TreeNodeViewImpl::set_allow_sorting(TreeNodeView* self, bool flag)
@@ -1570,39 +1670,91 @@ void TreeNodeViewImpl::set_allow_sorting(bool flag)
 {
   if (_tree.get_headers_visible())
     _tree.set_headers_clickable(flag);
-  if (flag && _tree_store)
+  
+  if (flag == false || !_tree_store)
+    return;
+    
+  if (!_sort_model)
+    _sort_model = Gtk::TreeModelSort::create(_tree_store);
+
+  for (std::size_t i = 0; i < _tree.get_columns().size(); ++i)
   {
-    if (!_sort_model)
-      _sort_model = Gtk::TreeModelSort::create(_tree_store);
-
-    const int ncols = _tree.get_columns().size();
-    for (int i = 0; i < ncols; ++i)
+    Gtk::TreeViewColumn* col = _tree.get_column(i);
+    Gtk::TreeModelColumnBase *mcol = _columns.columns[index_for_column(i)];
+    
+    if (col == NULL || mcol == NULL)
     {
-
-      Gtk::TreeViewColumn* col = _tree.get_column(i);
-      Gtk::TreeModelColumnBase *mcol = _columns.columns[index_for_column(i)];
-      if (get_owner()->get_column_type(i) == NumberWithUnitColumnType)
-      {
-        _sort_model->set_sort_func(*mcol, sigc::bind(sigc::ptr_fun(str_number_cmp), (Gtk::TreeModelColumn< Glib::ustring>*)mcol));
-      }
-      else if (mcol->type() == G_TYPE_STRING)
-      {
-        _sort_model->set_sort_func(*mcol, sigc::bind(sigc::ptr_fun(str_cmp), (Gtk::TreeModelColumn< Glib::ustring>*)mcol));
-      }
-
-
-
-      if (mcol && col)
-      {
-        col->signal_clicked().connect(sigc::bind(sigc::mem_fun(this, &TreeNodeViewImpl::header_clicked),mcol,col));
-      }
+      log_warning("Invalid column pointer[col:%s][mcol:%s]\n", col == NULL ? "NULL" : "NOT NULL", mcol == NULL ? "NULL" : "NOT NULL");
+      continue;
     }
 
-    // temporarily disable selection change signal, gtk emits it when setting model
-    _conn.disconnect();
-    _tree.set_model(_sort_model);
-    _conn = _tree.get_selection()->signal_changed().connect(sigc::mem_fun(dynamic_cast<TreeNodeView*>(owner), &TreeNodeView::changed));
+    col->signal_clicked().connect(sigc::bind(sigc::mem_fun(this, &TreeNodeViewImpl::header_clicked),mcol,col));
+    
+    //  Align columns text depending on the visualization type
+    float align = 0.0f;
+    
+    switch(get_owner()->get_column_type(i))
+    {
+      case StringColumnType:
+      case StringLTColumnType:
+      case IconColumnType:
+        align = 0.0f;
+        break;
+      case IntegerColumnType:
+      case LongIntegerColumnType:
+      case FloatColumnType:
+      case NumberWithUnitColumnType:
+        align = 1.0f;
+        break;
+      case CheckColumnType:
+      case TriCheckColumnType:
+        align = 0.5f;
+        break;
+    }      
+    
+    Glib::ListHandle<Gtk::CellRenderer*> renderers = col->get_cell_renderers();
+    
+    for (Glib::ListHandle<Gtk::CellRenderer*>::const_iterator iter = renderers.begin(); iter != renderers.end(); ++iter)
+      (*iter)->set_alignment(align, 0);
+    
+    //  Set the proper compare method to the sorting functions depending on the storage type
+    switch (mcol->type())
+    {
+      case G_TYPE_INT:
+        _sort_model->set_sort_func(*mcol, sigc::bind(sigc::ptr_fun(column_numeric_compare<int>), (Gtk::TreeModelColumn<int>*)mcol));
+        break;
+      case G_TYPE_LONG:
+        _sort_model->set_sort_func(*mcol, sigc::bind(sigc::ptr_fun(column_numeric_compare<long>), (Gtk::TreeModelColumn<long>*)mcol));
+        break;
+      case G_TYPE_INT64:
+        _sort_model->set_sort_func(*mcol, sigc::bind(sigc::ptr_fun(column_numeric_compare<int64_t>), (Gtk::TreeModelColumn<int64_t>*)mcol));
+        break;
+      case G_TYPE_UINT:
+        _sort_model->set_sort_func(*mcol, sigc::bind(sigc::ptr_fun(column_numeric_compare<unsigned int>), (Gtk::TreeModelColumn<unsigned int>*)mcol));
+        break;
+      case G_TYPE_ULONG:
+        _sort_model->set_sort_func(*mcol, sigc::bind(sigc::ptr_fun(column_numeric_compare<unsigned long>), (Gtk::TreeModelColumn<unsigned long>*)mcol));
+        break;
+      case G_TYPE_UINT64:
+        _sort_model->set_sort_func(*mcol, sigc::bind(sigc::ptr_fun(column_numeric_compare<uint64_t>), (Gtk::TreeModelColumn<uint64_t>*)mcol));
+        break;
+      case G_TYPE_FLOAT:
+      case G_TYPE_DOUBLE:
+        _sort_model->set_sort_func(*mcol, sigc::bind(sigc::ptr_fun(column_numeric_compare<double>), (Gtk::TreeModelColumn<double>*)mcol));
+        break;
+      case G_TYPE_STRING:
+        _sort_model->set_sort_func(*mcol, sigc::bind(sigc::ptr_fun(column_string_compare), (Gtk::TreeModelColumn<Glib::ustring>*)mcol, get_owner()->get_column_type(i)));
+        break;
+      default:
+        log_warning("Unknown column storage type");
+        break;
+    }
   }
+
+  // temporarily disable selection change signal, gtk emits it when setting model
+  _conn.disconnect();
+  _tree.set_model(_sort_model);
+  _conn = _tree.get_selection()->signal_changed().connect(sigc::mem_fun(dynamic_cast<TreeNodeView*>(owner), &TreeNodeView::changed));
 }
 
 void TreeNodeViewImpl::freeze_refresh(TreeNodeView* self, bool flag)
