@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -137,7 +137,7 @@ PythonContext::PythonContext(GRT *grt, const std::string &module_path)
   {
     PyObject *path = from_grt(grt::StringRef(base::Logger::log_filename()));
     PyDict_SetItemString(PyModule_GetDict(PyImport_AddModule("grt")), "logpath", path);
-    Py_DECREF(path);
+    Py_XDECREF(path);
   }
 
   PyEval_SaveThread();
@@ -307,7 +307,7 @@ void PythonContext::handle_grt_notification(const std::string &name, ObjectRef s
       Py_DECREF(res);
     }
     Py_DECREF(psender);
-    Py_DECREF(pinfo);
+    Py_XDECREF(pinfo);
     Py_DECREF(args);
   }
 }
@@ -358,6 +358,7 @@ void PythonContext::handle_notification(const std::string &name, void *sender, b
         info[s]= v.repr();
       }
     }
+    Py_DECREF(infodict);
   }  
 }
 
@@ -1044,6 +1045,71 @@ void PythonContext::set_unwrap_pyobject_func(PyObject *(*func)(PyObject *, PyObj
 }
 
 
+static boost::function<boost::signals2::connection (const boost::function<void ()>&)> run_once_when_idle;
+
+void PythonContext::set_run_once_when_idle(boost::function<boost::signals2::connection (const boost::function<void ()>&)> func)
+{
+  run_once_when_idle = func;
+}
+
+static void call_callback(AutoPyObject callback)
+{
+  WillEnterPython lock;
+  PyObject *args = PyTuple_New(0);
+  PyObject *r = PyObject_Call(callback, args, NULL);
+  Py_XDECREF(args);
+  if (r == NULL)
+    PythonContext::log_python_error("Exeption executing callback for run_from_main_thread:\n");
+  else
+    Py_DECREF(r);
+}
+
+static void delete_connection(void *conn, void *desc)
+{
+  delete reinterpret_cast<boost::signals2::connection*>(conn);
+}
+
+
+static PyObject *grt_run_from_main_thread(PyObject *self, PyObject *args)
+{
+  PyObject *object;
+  
+  if (!PyArg_ParseTuple(args, "O", &object))
+    return NULL;
+  
+  if (!PyCallable_Check(object))
+  {
+    PyErr_SetString(PyExc_ValueError, "notification observer argument must be a callable");
+    return NULL;
+  }
+
+  boost::signals2::connection conn = run_once_when_idle(boost::bind(call_callback, AutoPyObject(object)));
+
+  return PyCObject_FromVoidPtrAndDesc(new boost::signals2::connection(conn), (void*)call_callback, delete_connection);
+}
+
+
+static PyObject *grt_cancel_run_from_main_thread(PyObject *self, PyObject *args)
+{
+  PyObject *object;
+  
+  if (!PyArg_ParseTuple(args, "O:CObject", &object))
+    return NULL;
+
+  if (PyCObject_GetDesc(object) == (void*)call_callback)
+  {
+    boost::signals2::connection *conn = reinterpret_cast<boost::signals2::connection*>(PyCObject_AsVoidPtr(object));
+    if (conn && conn->connected())
+      conn->disconnect();
+    Py_INCREF(Py_None);
+    return Py_None;
+  }
+  
+  return NULL;
+}
+
+
+
 /** Register grt related functionality as a module
 
  Stuff made available in the module include:
@@ -1120,6 +1186,11 @@ static PyMethodDef GrtModuleMethods[] = {
 {"togrt", grt_wrap_pyobject, METH_VARARGS,
   "Wraps a Python object in a grt_PyObject GRT object, which you can then use to reference from GRT objects (such as GRT dicts and lists)"},
 
+{"run_from_main_thread", grt_run_from_main_thread, METH_VARARGS,
+  "Executes a Python callable from the main thread once, as soon as possible. Return value can be given to cancel_run_from_main_thread to cancel it's execution, if it hasn't executed yet."},
+
+{"cancel_run_from_main_thread", grt_cancel_run_from_main_thread, METH_VARARGS,
+  "Cancels execution of callback from main thread, which was previously scheduled with run_from_main_thread."},
 
 {NULL, NULL, 0, NULL}        /* Sentinel */
 };
@@ -1565,7 +1636,7 @@ ValueRef PythonContext::from_pyobject(PyObject *object, const grt::TypeSpec &exp
     {
       if (PyList_Check(object))
       {
-        grt::BaseListRef list(_grt);
+        grt::BaseListRef list(_grt, expected_type.content.type);
         
         for (Py_ssize_t c= PyList_Size(object), i= 0; i < c; i++)
         {
@@ -1699,7 +1770,6 @@ int PythonContext::run_buffer(const std::string &buffer, std::string *line_buffe
   {
     return 0; // continued line
   }
-
   if (!n && PyErr_Occurred() && PyErr_ExceptionMatches(PyExc_SyntaxError))
   {
     PyObject *excep;
@@ -1708,7 +1778,8 @@ int PythonContext::run_buffer(const std::string &buffer, std::string *line_buffe
     PyObject *trace;
     PyErr_Fetch(&excep, &value, &trace);
     message= PyTuple_GetItem(value, 0);
-    if (strstr(PyString_AsString(message), "unexpected EOF") ||
+    if (strstr(PyString_AsString(message), "expected an indented block") ||
+        strstr(PyString_AsString(message), "unexpected EOF") ||
         strncmp(PyString_AsString(message), "EOF", 3)==0)
     {
       Py_DECREF(excep);
@@ -1840,7 +1911,9 @@ int PythonContext::refresh()
   const std::vector<grt::Module*> &modules(_grt->get_modules());
   for (std::vector<grt::Module*>::const_iterator iter= modules.begin(); iter != modules.end(); ++iter)
   {
-    PyObject *r= PyObject_Call(_grt_module_class, Py_BuildValue("(s)", (*iter)->name().c_str()), NULL);
+    PyObject *arg = Py_BuildValue("(s)", (*iter)->name().c_str());
+    PyObject *r= PyObject_Call(_grt_module_class, arg, NULL);
+    Py_DECREF(arg);
 
     if (!r)
       PythonContext::log_python_error("Error refreshing grt modules");
