@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -55,6 +55,7 @@ namespace MySQL.GUI.Workbench.Plugins
     private bool headingCollapsed = false;
     private bool privPageWasActive = false; // Set when doing re-init.
     private bool updatingTable = false;
+    private bool updatingFkValues = false;
 
     #endregion
 
@@ -485,6 +486,7 @@ namespace MySQL.GUI.Workbench.Plugins
         columnsListModel.RefreshModel();
         indicesListModel.RefreshModel();
         fkListModel.RefreshModel();
+        tableEditorBE.load_trigger_sql();
 
         // partitioning
         if (tableEditorBE.get_partition_type() == null || tableEditorBE.get_partition_type() == "")
@@ -507,12 +509,8 @@ namespace MySQL.GUI.Workbench.Plugins
 
         refreshPartitioningList();
 
-        // table inserts editor
-        if (null != insertsRecordsetView)
-        {
+        if (insertsRecordsetView != null)
           insertsRecordsetView.Model = tableEditorBE.get_inserts_model();
-          insertsRecordsetView.Model.refresh();
-        }
       }
       finally
       {
@@ -575,7 +573,7 @@ namespace MySQL.GUI.Workbench.Plugins
       }
       else if (mainTabControl.SelectedTab == insertsTabPage)
       {
-        ActiveControl = insertsRecordsetView.GridView;
+      //XXX!  ActiveControl = insertsRecordsetView.GridView;
         insertsRecordsetView.Model.refresh();
       }
     }
@@ -1469,61 +1467,79 @@ namespace MySQL.GUI.Workbench.Plugins
 
     private void fkTreeView_SelectionChanged(object sender, EventArgs e)
     {
-      // remove old virtual value events
-      if (fkColumnsTreeView.Model != null)
-        fkColumnsListModel.DetachEvents();
+      updatingFkValues = true;
 
-      // if a new node was selected, create an inspector for it
-      if (fkTreeView.SelectedNode != null && fkTreeView.SelectedNode.NextNode != null)
+      try
       {
-        GrtListNode node = fkTreeView.SelectedNode.Tag as GrtListNode;
+        // Remove old virtual value events.
+        if (fkColumnsTreeView.Model != null)
+          fkColumnsListModel.DetachEvents();
 
-        if (node != null)
+        // if a new node was selected, create an inspector for it
+        if (fkTreeView.SelectedNode != null && fkTreeView.SelectedNode.NextNode != null)
         {
-          String text;
+          GrtListNode node = fkTreeView.SelectedNode.Tag as GrtListNode;
 
-          fkCommentText.Enabled = true;
-          onDeleteActionComboBox.Enabled = true;
-          onUpdateActionComboBox.Enabled = true;
+          if (node != null)
+          {
+            String text;
 
-          // select the fk
-          tableEditorBE.get_fks().select_fk(node.NodeId);
+            fkCommentText.Enabled = true;
+            onDeleteActionComboBox.Enabled = true;
+            onUpdateActionComboBox.Enabled = true;
 
-          // create new inspector model for the selected value, pass valueNodeTextBox so virtual value events can be attached
-          fkColumnsListModel = new DbMysqlTableFkColumnListModel(fkColumnsTreeView,
-            tableEditorBE.get_fks().get_columns(),
-            columnEnabledFkNodeControl, columnFkNodeControl,
-            targetColumnFkNodeControl, tableEditorBE);
+            // select the fk
+            tableEditorBE.get_fks().select_fk(node.NodeId);
 
-          tableEditorBE.get_fks().get_field(node.NodeId, (int)MySQL.Grt.Db.FKConstraintListWrapper.FKConstraintListColumns.OnDelete, out text);
-          if (String.IsNullOrEmpty(text))
+            // create new inspector model for the selected value, pass valueNodeTextBox so virtual value events can be attached
+            fkColumnsListModel = new DbMysqlTableFkColumnListModel(fkColumnsTreeView,
+              tableEditorBE.get_fks().get_columns(),
+              columnEnabledFkNodeControl, columnFkNodeControl,
+              targetColumnFkNodeControl, tableEditorBE);
+
+            tableEditorBE.get_fks().get_field(node.NodeId, (int)FKConstraintListWrapper.FKConstraintListColumns.OnDelete, out text);
+            if (String.IsNullOrEmpty(text))
+              onDeleteActionComboBox.SelectedIndex = -1;
+            else
+              onDeleteActionComboBox.SelectedItem = text;
+
+            tableEditorBE.get_fks().get_field(node.NodeId, (int)FKConstraintListWrapper.FKConstraintListColumns.OnUpdate, out text);
+            if (String.IsNullOrEmpty(text))
+              onUpdateActionComboBox.SelectedIndex = -1;
+            else
+              onUpdateActionComboBox.SelectedItem = text;
+
+            tableEditorBE.get_fks().get_field(node.NodeId, (int)FKConstraintListWrapper.FKConstraintListColumns.Comment, out text);
+            fkCommentText.Text = text;
+
+            tableEditorBE.get_fks().get_field(node.NodeId, (int)FKConstraintListWrapper.FKConstraintListColumns.Index, out text);
+
+            int value = 0;
+            tableEditorBE.get_fks().get_field(node.NodeId, (int)FKConstraintListWrapper.FKConstraintListColumns.ModelOnly, out value);
+            fkModelOnlyCheck.Checked = value != 0;
+            fkModelOnlyCheck.Enabled = true;
+
+            // assign model to treeview
+            fkColumnsTreeView.Model = fkColumnsListModel;
+          }
+          else
+          {
+            fkColumnsTreeView.Model = null;
             onDeleteActionComboBox.SelectedIndex = -1;
-          else
-            onDeleteActionComboBox.SelectedItem = text;
-
-          tableEditorBE.get_fks().get_field(node.NodeId, (int)MySQL.Grt.Db.FKConstraintListWrapper.FKConstraintListColumns.OnUpdate, out text);
-          if (String.IsNullOrEmpty(text))
             onUpdateActionComboBox.SelectedIndex = -1;
-          else
-            onUpdateActionComboBox.SelectedItem = text;
-
-          tableEditorBE.get_fks().get_field(node.NodeId, (int)MySQL.Grt.Db.FKConstraintListWrapper.FKConstraintListColumns.Comment, out text);
-          fkCommentText.Text = text;
-
-          tableEditorBE.get_fks().get_field(node.NodeId, (int)MySQL.Grt.Db.FKConstraintListWrapper.FKConstraintListColumns.Index, out text);
-
-          int value= 0;
-          tableEditorBE.get_fks().get_field(node.NodeId, (int)MySQL.Grt.Db.FKConstraintListWrapper.FKConstraintListColumns.ModelOnly, out value);
-          fkModelOnlyCheck.Checked = value != 0;
-          fkModelOnlyCheck.Enabled = true;
-
-          // assign model to treeview
-          fkColumnsTreeView.Model = fkColumnsListModel;
+            fkCommentText.Text = "";
+            fkIndexLabel.Text = "";
+            fkModelOnlyCheck.Checked = false;
+            fkModelOnlyCheck.Enabled = false;
+            fkCommentText.Enabled = false;
+            onDeleteActionComboBox.Enabled = false;
+            onUpdateActionComboBox.Enabled = false;
+          }
         }
         else
         {
           fkColumnsTreeView.Model = null;
-          onDeleteActionComboBox.SelectedIndex= -1;
+          onDeleteActionComboBox.SelectedIndex = -1;
           onUpdateActionComboBox.SelectedIndex = -1;
           fkCommentText.Text = "";
           fkIndexLabel.Text = "";
@@ -1534,18 +1550,9 @@ namespace MySQL.GUI.Workbench.Plugins
           onUpdateActionComboBox.Enabled = false;
         }
       }
-      else
+      finally
       {
-        fkColumnsTreeView.Model = null;
-        onDeleteActionComboBox.SelectedIndex = -1;
-        onUpdateActionComboBox.SelectedIndex = -1;
-        fkCommentText.Text = "";
-        fkIndexLabel.Text = "";
-        fkModelOnlyCheck.Checked = false;
-        fkModelOnlyCheck.Enabled = false;
-        fkCommentText.Enabled = false;
-        onDeleteActionComboBox.Enabled = false;
-        onUpdateActionComboBox.Enabled = false;
+        updatingFkValues = false;
       }
     }
 
@@ -1558,7 +1565,7 @@ namespace MySQL.GUI.Workbench.Plugins
           GrtListNode listNode = node.Tag as GrtListNode;
 
           tableEditorBE.get_fks().set_field(listNode.NodeId,
-            (int)MySQL.Grt.Db.FKConstraintListWrapper.FKConstraintListColumns.ModelOnly,
+            (int)FKConstraintListWrapper.FKConstraintListColumns.ModelOnly,
             fkModelOnlyCheck.Checked ? 1 : 0);
         }
       }
@@ -1592,30 +1599,44 @@ namespace MySQL.GUI.Workbench.Plugins
 
     private void onUpdateActionComboBox_SelectedIndexChanged(object sender, EventArgs e)
     {
-      if (onUpdateActionComboBox.SelectedItem != null)
+      if (!updatingFkValues && onUpdateActionComboBox.SelectedItem != null)
       {
-        foreach (TreeNodeAdv node in fkTreeView.SelectedNodes)
-        {
-          GrtListNode listNode = node.Tag as GrtListNode;
+        // Always assigned, otherwise the combobox would be disabled.
+        GrtListNode node = fkTreeView.SelectedNode.Tag as GrtListNode;
 
-          tableEditorBE.get_fks().set_field(listNode.NodeId,
-            (int)MySQL.Grt.Db.FKConstraintListWrapper.FKConstraintListColumns.OnUpdate,
-            onUpdateActionComboBox.SelectedItem.ToString());
+        if (!tableEditorBE.get_fks().set_field(node.NodeId,
+          (int)FKConstraintListWrapper.FKConstraintListColumns.OnUpdate,
+          onUpdateActionComboBox.SelectedItem.ToString()))
+        {
+          // The change was rejected. Revert the combobox to its original value.
+          string text;
+          tableEditorBE.get_fks().get_field(node.NodeId, (int)FKConstraintListWrapper.FKConstraintListColumns.OnUpdate, out text);
+          if (String.IsNullOrEmpty(text))
+            onUpdateActionComboBox.SelectedIndex = -1;
+          else
+            onUpdateActionComboBox.SelectedItem = text;
         }
       }
     }
 
     private void onDeleteActionComboBox_SelectedIndexChanged(object sender, EventArgs e)
     {
-      if (onDeleteActionComboBox.SelectedItem != null)
+      if (!updatingFkValues && onDeleteActionComboBox.SelectedItem != null)
       {
-        foreach (TreeNodeAdv node in fkTreeView.SelectedNodes)
-        {
-          GrtListNode listNode = node.Tag as GrtListNode;
+        // Always assigned, otherwise the combobox would be disabled.
+        GrtListNode node = fkTreeView.SelectedNode.Tag as GrtListNode;
 
-          tableEditorBE.get_fks().set_field(listNode.NodeId,
-            (int)MySQL.Grt.Db.FKConstraintListWrapper.FKConstraintListColumns.OnDelete,
-            onDeleteActionComboBox.SelectedItem.ToString());
+        if (!tableEditorBE.get_fks().set_field(node.NodeId,
+          (int)FKConstraintListWrapper.FKConstraintListColumns.OnDelete,
+          onDeleteActionComboBox.SelectedItem.ToString()))
+        {
+          // The change was rejected. Revert the combobox to its original value.
+          string text;
+          tableEditorBE.get_fks().get_field(node.NodeId, (int)FKConstraintListWrapper.FKConstraintListColumns.OnDelete, out text);
+          if (String.IsNullOrEmpty(text))
+            onDeleteActionComboBox.SelectedIndex = -1;
+          else
+            onDeleteActionComboBox.SelectedItem = text;
         }
       }
     }
@@ -1627,7 +1648,7 @@ namespace MySQL.GUI.Workbench.Plugins
         GrtListNode listNode = node.Tag as GrtListNode;
 
         tableEditorBE.get_fks().set_field(listNode.NodeId,
-          (int)MySQL.Grt.Db.FKConstraintListWrapper.FKConstraintListColumns.Comment,
+          (int)FKConstraintListWrapper.FKConstraintListColumns.Comment,
           fkCommentText.Text);
       }
     }
@@ -1643,7 +1664,11 @@ namespace MySQL.GUI.Workbench.Plugins
       if (insertsRecordsetView == null)
       {
         insertsRecordsetView = new MySQL.Grt.Db.RecordsetView();
-        insertsRecordsetView.Embed(insertsTabPage, tableEditorBE.get_inserts_model());
+        insertsRecordsetView.SetupRecordset(tableEditorBE.get_inserts_model());
+        Control panel = tableEditorBE.get_inserts_panel(insertsRecordsetView.GridView);
+        insertsTabPage.Controls.Add(panel);
+        panel.Parent = insertsTabPage;
+        panel.Dock = DockStyle.Fill;
       }
     }
 
