@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -120,12 +120,20 @@ void DbMySQLSQLExport::set_option(const std::string& name, const std::string& va
 {
   if(name.compare("OutputFileName") == 0)
     _output_filename= value;
+  else if(name.compare("OutputScriptHeader") == 0)
+    _output_header= value;
 }
 
 void DbMySQLSQLExport::set_db_options_for_version(const GrtVersionRef &version)
 {
   SQLGeneratorInterfaceImpl *diffsql_module= dynamic_cast<SQLGeneratorInterfaceImpl*>(version.get_grt()->get_module("DbMySQL"));
-  _db_options = diffsql_module->getTraitsForServerVersion(version->majorNumber(), version->minorNumber(), version->releaseNumber());
+  if (diffsql_module != NULL)
+    _db_options = diffsql_module->getTraitsForServerVersion(version->majorNumber(), version->minorNumber(), version->releaseNumber());
+}
+
+void DbMySQLSQLExport::set_db_options(grt::DictRef &db_options)
+{
+    _db_options = db_options;
 }
 
 grt::StringListRef convert_string_vector_to_grt_list(grt::GRT *grt, const std::vector<std::string>& v)
@@ -180,7 +188,7 @@ grt::DictRef DbMySQLSQLExport::get_options_as_dict(grt::GRT *grt)
   options.set("GenerateWarnings", grt::IntegerRef(_gen_warnings ? 1 : 0));
   options.set("GenerateCreateIndex", grt::IntegerRef(_gen_create_index ? 1 : 0));
   options.set("NoUsersJustPrivileges", grt::IntegerRef(_no_users_just_privileges ? 1 : 0));
-  options.set("NoViewPlaceholders", grt::IntegerRef(_no_users_just_privileges ? 1 : 0));
+  options.set("NoViewPlaceholders", grt::IntegerRef(_no_view_placeholders ? 1 : 0));
   options.set("GenerateInserts", grt::IntegerRef(_gen_inserts ? 1 : 0));
   options.set("NoFKForInserts", grt::IntegerRef(_no_FK_for_inserts ? 1 : 0));
   options.set("TriggersAfterInserts", grt::IntegerRef(_triggers_after_inserts ? 1 : 0));
@@ -189,6 +197,8 @@ grt::DictRef DbMySQLSQLExport::get_options_as_dict(grt::GRT *grt)
   options.set("SkipForeignKeys", grt::IntegerRef(_skip_foreign_keys ? 1 : 0));
   options.set("SkipFKIndexes", grt::IntegerRef(_skip_fk_indexes ? 1 : 0));
 
+  options.set("OutputScriptHeader", grt::StringRef(_output_header));
+
   std::vector<std::string> schemata_names;
   db_mysql_CatalogRef cat(get_model_catalog());
 
@@ -303,7 +313,8 @@ ValueRef DbMySQLSQLExport::export_task(grt::GRT* grt, grt::StringRef)
       return grt::StringRef("\nSQL Script Export Error: SQL Script Export Module Returned Error");
     }
     
-    _export_sql_script= options.get_string("OutputScript");
+    _export_sql_script= options.get_string("OutputScriptHeader") + options.get_string("OutputScript");
+
     if (!_output_filename.empty())
     {
       g_file_set_contents(_output_filename.c_str(), 
