@@ -1,4 +1,4 @@
-# Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -18,6 +18,9 @@
 from workbench.db_utils import QueryError, escape_sql_string
 from wb_common import PermissionDeniedError
 
+import mforms
+from workbench.log import log_error
+
 
 class WBSecurityValidationError(Exception):
     pass
@@ -26,6 +29,17 @@ class WBSecurityValidationError(Exception):
 LIST_ACCOUNTS_QUERY = "SELECT User, Host FROM mysql.user ORDER BY User"
 LIST_SCHEMAS_QUERY = "SHOW DATABASES"
 
+ZOMBIE_SCHEMA_PRIVS_QUERY = "SELECT d.User, d.Host, d.Db FROM mysql.db AS d LEFT JOIN mysql.user AS u ON d.User = u.user AND d.Host = u.Host WHERE u.User IS NULL"
+ZOMBIE_TABLE_PRIVS_QUERY = "SELECT t.User, t.Host, t.Db, t.Table_name FROM mysql.tables_priv AS t LEFT JOIN mysql.user AS u ON t.User = u.user AND t.Host = u.Host WHERE u.User IS NULL"
+ZOMBIE_COLUMN_PRIVS_QUERY = "SELECT c.User, c.Host, c.Db, c.Table_name, c.Column_name FROM mysql.columns_priv AS c LEFT JOIN mysql.user AS u ON c.User = u.user AND c.Host = u.Host WHERE u.User IS NULL"
+ZOMBIE_PROCS_PRIVS_QUERY = "SELECT p.User, p.Host, p.Db, p.Routine_name, p.Routine_type FROM mysql.procs_priv AS p LEFT JOIN mysql.user AS u ON p.User = u.user AND p.Host = u.Host WHERE u.User IS NULL"
+
+DROP_ZOMBIE_SCHEMA_PRIVS_QUERY = "DELETE FROM mysql.db WHERE User = '%(user)s' AND Host = '%(host)s'"
+DROP_ZOMBIE_TABLE_PRIVS_QUERY = "DELETE FROM mysql.tables_priv WHERE User = '%(user)s' AND Host = '%(host)s'"
+DROP_ZOMBIE_COLUMN_PRIVS_QUERY =  "DELETE FROM mysql.columns_priv WHERE User = '%(user)s' AND Host = '%(host)s'"
+DROP_ZOMBIE_PROCS_PRIVS_QUERY =  "DELETE FROM mysql.procs_priv WHERE User = '%(user)s' AND Host = '%(host)s'"
+
+
 GET_ACCOUNT_QUERY = "SELECT * FROM mysql.user WHERE User='%(user)s' AND Host='%(host)s' ORDER BY User, Host"
 
 GET_ACCOUNT_SCHEMA_PRIVS_QUERY = "SELECT * FROM mysql.db WHERE User='%(user)s' AND Host='%(host)s' ORDER BY Db"
@@ -209,6 +223,7 @@ class AdminSecurity(object):
         self.ctrl_be = ctrl_be
         self._accounts = []
         self._schema_names = []
+        self._zombie_privs = {} # (user, host): [objects]
 
         self._account_info_cache = {}  # A mapping with the form 'user@host' -> AdminAccount instance for that user
         self._schema_privileges_cache = {}
@@ -312,9 +327,36 @@ class AdminSecurity(object):
                 host = result.stringByName("Host")
                 accounts.append((user, host))
 
+
+        # Get a list of invalid privileges
+        def get_zombies(query, fields):
+            try:
+                result = self.ctrl_be.exec_query(query)
+            except Exception, e:
+                log_error("Could not get list of invalid privs: %s\nQuery: %s\n" % (e, query))
+                return []
+            privs = []
+            while result and result.nextRow():
+                user = result.stringByName("User")
+                host = result.stringByName("Host")
+                parts = [result.stringByName(f) for f in fields]
+                privs.append(((user, host), ".".join(parts)))
+            return privs
+
         # Sort list of accounts by User and Host
         # User is sorted alphabetically, Host is sorted by specificity in addition to alpha
         # (most specific items first)
+        privs = get_zombies(ZOMBIE_SCHEMA_PRIVS_QUERY, ["Db"])
+        privs += get_zombies(ZOMBIE_TABLE_PRIVS_QUERY, ["Db", "Table_name"])
+        privs += get_zombies(ZOMBIE_COLUMN_PRIVS_QUERY, ["Db", "Table_name", "Column_name"])
+        privs += get_zombies(ZOMBIE_PROCS_PRIVS_QUERY, ["Db", "Routine_name"])
+        zombies = {}
+        for account, priv in privs:
+            if account not in zombies:
+                zombies[account] = []
+            zombies[account].append(priv)
+        self._zombie_privs = zombies
+        accounts += zombies.keys()
 
         accounts.sort(acl_compare)
 
@@ -392,19 +434,53 @@ class AdminSecurity(object):
         return acct
 
 
+    def is_zombie(self, user, host):
+        return self._zombie_privs.has_key((user,host))
+
+
+    def get_zombie_privs(self, user, host):
+        return self._zombie_privs.get((user,host), None)
+
+
+    def wipe_zombie(self, user, host):
+        try:
+            self.ctrl_be.exec_query(DROP_ZOMBIE_SCHEMA_PRIVS_QUERY % {'user':escape_sql_string(user), 'host':escape_sql_string(host)})
+        except Exception, e:
+            log_error("Could not delete schema privileges for %s@%s: %s\n" % (user, host, e))
+        try:
+            self.ctrl_be.exec_query(DROP_ZOMBIE_TABLE_PRIVS_QUERY % {'user':escape_sql_string(user), 'host':escape_sql_string(host)})
+        except Exception, e:
+            log_error("Could not delete table privileges for %s@%s: %s\n" % (user, host, e))
+        try:
+            self.ctrl_be.exec_query(DROP_ZOMBIE_COLUMN_PRIVS_QUERY % {'user':escape_sql_string(user), 'host':escape_sql_string(host)})
+        except Exception, e:
+            log_error("Could not delete column privileges for %s@%s: %s\n" % (user, host, e))
+        try:
+            self.ctrl_be.exec_query(DROP_ZOMBIE_PROCS_PRIVS_QUERY % {'user':escape_sql_string(user), 'host':escape_sql_string(host)})
+        except Exception, e:
+            log_error("Could not delete procs privileges for %s@%s: %s\n" % (user, host, e))
+
+
+
+
+
     def async_get_account(self, callback, name, host):
-        key = name+"@"+host
+        try:
+          key = name+"@"+host
 
-        if self._account_info_cache.has_key(key):
-            callback(self._account_info_cache[key])
-            return
+          if self._account_info_cache.has_key(key):
+              callback(self._account_info_cache[key])
+              return
 
-        account = AdminAccount(self)
-        account.load(name, host)
+          account = AdminAccount(self)
+          account.load(name, host)
 
-        self._account_info_cache[key] = account
+          self._account_info_cache[key] = account
 
-        callback(account)
+          callback(account)
+        except:
+          mforms.Utilities.driver_shutdown()
+          raise
 
 
 
@@ -493,8 +569,6 @@ class AdminUserDbPrivs(object):
         # workaround for server bug
         self._owner.ctrl_be.exec_sql("use mysql")
 
-        handled_entries = set()
-
         for deleted, entry in [(True, e) for e in self._deleted_entries] + [(False, e) for e in self.entries]:
 
             fields = { "user":escape_sql_string(self._owner.username),
@@ -554,6 +628,8 @@ class AdminAccount(object):
     max_user_connections = 0
     auth_plugin = None
     auth_string = None
+    old_authentication = False
+    blank_password = False
 
     _global_privs = set()
     _custom_checked_privs = set()
@@ -872,6 +948,8 @@ class AdminAccount(object):
         self.password_expired = False
         if self._owner.has_password_expired:
             self.password_expired = result.stringByName("password_expired") == 'Y'
+        self.old_authentication = len(result.stringByName("password")) == 16
+        self.blank_password = len(result.stringByName("password")) == 0
 
         self._global_privs = set()
         for priv in self._owner.global_privilege_names:
@@ -927,3 +1005,38 @@ class AdminAccount(object):
         self._orig_global_privs = self._global_privs.copy()
 
         self.schema_privs.load()
+
+    def upgrade_password_format(self):
+        queries = []
+        if self.password != self.confirm_password:
+            raise WBSecurityValidationError("The new password and its confirmation don't match. Please re-enter them.")
+
+        queries.append("use mysql")
+
+        if not self.host:
+            raise WBSecurityValidationError("Host name must not be blank")
+
+        fields = {
+            "user" : escape_sql_string(self.username) or "NULL",
+            "host" : escape_sql_string(self.host) or "",
+            "password" : escape_sql_string(self.password or ""),
+        }
+
+        queries.append("SET old_passwords = 0")        
+        if self._owner.ctrl_be.target_version and self._owner.ctrl_be.target_version.is_supported_mysql_version_at_least(5, 5, 7):
+            queries.append("UPDATE mysql.user SET plugin = 'mysql_native_password' WHERE user = '%(user)s' AND host = '%(host)s'" % fields)
+        queries.append("FLUSH PRIVILEGES")
+        queries.append("SET PASSWORD FOR '%(user)s'@'%(host)s' = PASSWORD('%(password)s')" % fields)
+        queries.append("FLUSH PRIVILEGES")
+        
+        action = "changing"
+        for query in queries:
+            try:
+                self._owner.ctrl_be.exec_sql(query)
+            except QueryError, e:
+                if e.error == 1142:
+                    raise Exception("Error %s account %s@%s: Insufficient rights to perform operation"%(action, self.username, self.host))
+                else:
+                    raise Exception("Error %s account %s@%s: %s"%(action, self.username, self.host, e.errortext or e))
+            except Exception, e:
+                raise Exception("Error %s account %s@%s: %s"%(action, self.username, self.host, e))
